#!/usr/bin/env python
############################
# GK2A L1B data Processing sample code
#
# This program extract variables information and
# image_pixel_values data(Albedo) of
# GK2A Visible Ch.4 Level 1B data in netCDF4 format
#
# Input  : GK2A L1B file [sample file : VI004/fd010ge] (netCDF4)
#		   GK2A calibration table(ASCII)
# Output : sample_output.txt (ASCII)
# 		   sample_image_pixel_data.npy (numpy format)
############################
# 1.Library import
############################
import netCDF4
import numpy

############################
# 2. GK2A sample data file read
############################
input_ncfile_path = 'gk2a_ami_le1b_vi004_fd010ge_201901260310.nc'

CT_path='./calibration_table/'

output_txt_path='sample_output.txt'
output_npy_path='sample_image_pixel_data.npy'


############################
# 3. image_pixel_values DQF processing
############################
ncfile = netCDF4.Dataset(input_ncfile_path,'r',format='netcdf4')

ipixel=ncfile.variables['image_pixel_values']
ipixel_process = ipixel[:] 


############################
# 4. image_pixel_values DQF processing
############################
number_of_error_pixels = ipixel.getncattr('number_of_error_pixels')
if (number_of_error_pixels > 0):
	ipixel_process[ipixel_process>49151] = 0 #set error pixel's value to 0

############################
# 5. image_pixel_values Bit Size per pixel masking
############################
channel=ipixel.getncattr('channel_name')
if ((channel == 'VI004') or (channel == 'VI005') or (channel == 'NR016')):
	mask = 0b0000011111111111 #11bit mask
elif ((channel == 'VI006') or (channel == 'NR013') or (channel == 'WV063')):
	mask = 0b0000111111111111 #12bit mask
elif (channel == 'SW038'):
	mask = 0b0011111111111111 #14bit mask
else:
	mask = 0b0001111111111111 #13bit mask
	
ipixel_process_masked=numpy.bitwise_and(ipixel_process,mask)

############################
#6.image pixel value -> Albedo/Brightness Temperature
############################
AL_postfix='_cal_alb.txt'
BT_postfix='_cal_bt.txt'
if (channel[0:2] == 'VI') or (channel[0:2] == 'NR'):
	calibration_table=numpy.loadtxt(CT_path+channel+AL_postfix,'float64')
else:
	calibration_table=numpy.loadtxt(CT_path+channel+BT_postfix,'float64')

ipixel_process_masked_converted=calibration_table[ipixel_process_masked] # pixel data : table value / 1:1 matching
############################
#7. Data Output (ASCII format)
############################
output_txt=open(output_txt_path,'wt')

#print variable keys
var_keys = ncfile.variables.keys()
output_txt.write("number of variable keys : %s\n" %len(var_keys) )
for i in range(len(var_keys)):
	output_txt.write("%s\t%s\t%s\t%s\n" %(i,var_keys[i],ncfile.variables[var_keys[i]].dimensions,ncfile.variables[var_keys[i]].dtype))
output_txt.close()

############################
#8. Data Output (numpy format)
############################
numpy.save(output_npy_path,ipixel_process_masked_converted,False,False)


############################
#End of Program
############################